//
//  clebsch-gordan.c
//  
//
//  Created by Vincent Mathieu on 6/13/18.
//
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


int isfrac(double x){
  int val = 1;
  double intpart;
  if( modf(x, &intpart) == 0.0){ val = 0;}
  return val;
}


long int factorial(int n)
{
  if(n > 1)
    return n * factorial(n - 1);
  else
    return 1;
}


double clebsch(double j1, double j2, double j3, double m1, double m2){
  double cg = 0.0;
  double m3 = m1+m2;
  
  // check input
  if (isfrac(j1+j2+j3) || isfrac(j1+m1) || isfrac(j2+m2) || isfrac(j3+m3) ||
      isfrac(j3-j1-m2) || isfrac(j3-j2+m1)) {
    return 0.0;
  }
  
 
  // Check for conditions that give CG = 0.
  if ( j3 < fabs(j1-j2) || j3 < fabs(j1-j2)||
      fabs(m1) > j1 || fabs(m2) > j2 || fabs(m3) > j3 ) {
    return 0.0;
  }

  // Compute the Clebsch-Gordan coefficient
  cg = sqrt( (2*j3+1)/ factorial( round(j1+j2+j3+1) ) );
  cg = cg * sqrt( factorial(round(j1+j2-j3))*factorial(round(j2+j3-j1))*factorial(round(j3+j1-j2))   );
  cg = cg * sqrt( factorial(round(j1+m1))*factorial(round(j1-m1)) );
  cg = cg * sqrt( factorial(round(j2+m2))*factorial(round(j2-m2)) );
  cg = cg * sqrt( factorial(round(j3+m3))*factorial(round(j3-m3)) );
  
  double sum = 0.0, term;
  for (int k = 0; k < 99; k++) {
    if (j1+j2-j3-k < 0) continue;
    if (j3-j1-m2+k < 0) continue;
    if (j3-j2+m1+k < 0) continue;
    if (j1-m1-k    < 0) continue;
    if (j2+m2-k    < 0) continue;
    term = factorial(round(j1+j2-j3-k)) * factorial(round(j3-j1-m2+k)) ;
    term = term * factorial(round(j3-j2+m1+k)) * factorial(round(j1-m1-k));
    term = term * factorial(round(j2+m2-k)) * factorial(k);
    term = term * pow(-1, k);
    sum = sum + 1./term;
  }
  
  cg = cg * sum;
  
  return cg;
}

